/*******************************************************************
Copyright 2006 John Nikolakis

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software  Foundation;  either version 2  of the License, or
(at your option) any later version.

This  program is  distributed  in the  hope that it  will be useful,
but  WITHOUT  ANY  WARRANTY;  without even  the implied  warranty of
MERCHANTABILITY  or  FITNESS FOR A PARTICULAR PURPOSE.  See  the GNU
General Public License for more details.

You should  have received  a copy of the  GNU General Public License
along  with  this  program; if  not,  write  to the  Free  Software
Foundation, Inc.,  51  Franklin  Street,  Fifth  Floor,  Boston,
MA 02110-1301, USA 
*******************************************************************/

using System;
using System.Collections.Generic;
using System.Text;
using System.IO;
using System.Text.RegularExpressions;
using System.Xml;

/** ToDo List
* it has to check if the folders in home directory and bg.png is there
* Check if ImageMagik is installed
* If folder.* exists put folder frame as background (make it optional by a switch e.g. --FolderBackground)
*/
namespace ConsoleApplication1 {
    class Program {
        //Calculates the MD5 of a string
        static string MD5(string filename){
            System.Security.Cryptography.MD5CryptoServiceProvider x = new System.Security.Cryptography.MD5CryptoServiceProvider();
            byte[] bs = System.Text.Encoding.UTF8.GetBytes(filename);
            bs = x.ComputeHash(bs);
            System.Text.StringBuilder s = new System.Text.StringBuilder();
            foreach (byte b in bs){
                s.Append(b.ToString("x2").ToLower());
            }
            return s.ToString();
        }

        static void AppendXmlFile(string FolderPath, string PhotoPath, string XMLPath){
            Uri PhotoURI = new Uri(PhotoPath);
            string PhotoAbsolute = PhotoURI.AbsoluteUri;
            
            Uri FolderURI = new Uri(FolderPath);
            string XmlDocFileName = FolderURI.AbsoluteUri;
            string LastLetter = XmlDocFileName[XmlDocFileName.Length - 1] + " ";
            if (String.Compare(LastLetter, "/ ") == 0){
            	XmlDocFileName = XmlDocFileName.Substring(0, XmlDocFileName.Length - 1);
            }
            int LastSlash = XmlDocFileName.LastIndexOf(@"/");
            string FolderName = XmlDocFileName.Substring(LastSlash + 1, XmlDocFileName.Length - LastSlash -1);
            FolderName = Regex.Replace(FolderName, "/", "%2F");
            FolderName = Regex.Replace(FolderName, "&", "%26");
            FolderName = Regex.Replace(FolderName, ",", "%2C");
			FolderName = Regex.Replace(FolderName, "@", "%40");

            XmlDocFileName = XmlDocFileName.Substring(0, LastSlash);
            XmlDocFileName = Regex.Replace(XmlDocFileName, "/", "%2F");
            XmlDocFileName = Regex.Replace(XmlDocFileName, "&", "%26");
            XmlDocFileName = Regex.Replace(XmlDocFileName, ",", "%2C");
			XmlDocFileName = Regex.Replace(XmlDocFileName, "@", "%40");
            XmlDocFileName += ".xml";
            
            //Calculate UnixTimeStamp
            TimeSpan ts = (DateTime.UtcNow - new DateTime(1970, 1, 1, 0, 0, 0));
            string unixTimeStamp = ts.TotalSeconds.ToString("0");
			
			string OriginalFile = XMLPath + XmlDocFileName;
    		string TempFile = "temp.xml";
    		
			if (File.Exists(OriginalFile)){
				if (File.Exists(TempFile)){
					File.Delete(@TempFile);
				}
				File.Copy(OriginalFile, TempFile);
				
				XmlDocument xmlDocFile = new XmlDocument();
				xmlDocFile.Load(TempFile);
				
				// Create attributes
				XmlAttribute CustomIconXML = xmlDocFile.CreateAttribute("custom_icon");
				CustomIconXML.Value = PhotoAbsolute;
				XmlAttribute TimeStampXML = xmlDocFile.CreateAttribute("timestamp");
				TimeStampXML.Value = unixTimeStamp;
				
				int FolderNameFound = 0;
				
				XmlNodeList lstFiles = xmlDocFile.GetElementsByTagName("file");
				for (int i = 0; i < lstFiles.Count; i++) {
					// Get a list of the attributes of the current element
					XmlAttributeCollection curAttributes = lstFiles[i].Attributes;
					
					// Check if the current name is name of the folder
					if(curAttributes["name"].InnerText.Equals(FolderName)) {
						// add the new attributes
						((XmlElement)(lstFiles[i])).SetAttributeNode(CustomIconXML);
						((XmlElement)(lstFiles[i])).SetAttributeNode(TimeStampXML);
						
						// Save File
						xmlDocFile.Save(TempFile);
						
						FolderNameFound = 1;
						
						File.Delete(@OriginalFile);
						File.Copy(TempFile, OriginalFile);
						
						break;
					}
				}
				if (FolderNameFound < 1){
					XmlAttribute NameXML = xmlDocFile.CreateAttribute("name");
					NameXML.Value = FolderName;
				
					XmlElement elmXML = xmlDocFile.CreateElement("file");
				
					elmXML.SetAttributeNode(NameXML);
					elmXML.SetAttributeNode(TimeStampXML);
					elmXML.SetAttributeNode(CustomIconXML);
				
					xmlDocFile.DocumentElement.AppendChild(elmXML);
				
					xmlDocFile.Save(TempFile);
					File.Delete(@OriginalFile);
					File.Copy(TempFile, OriginalFile);
				}
				File.Delete(@TempFile);
			} else {
				//Create XML File Contents
	            string XMLData = "<?xml version=\"1.0\"?>\n<directory>";
	            XMLData += "<file name=\"" + FolderName + "\" timestamp=\"" + unixTimeStamp + "\" custom_icon=\"" + PhotoAbsolute + "\"/>";
	            XMLData += "</directory>\n";
	            
	            //Write XML File
	            TextWriter XMLFile = new StreamWriter(OriginalFile);
	            XMLFile.WriteLine(XMLData);
	            XMLFile.Close();
			}
        }
        
        static string FileToUse(string FileName, string Folder, string ThumbnailsPath){
        	string[] ImageAcceptedExtensions = {"jpg", "JPG", "jpeg", "JPEG", "gif", "GIF", "png", "PNG", "tiff", "TIFF"};
        	int PositionOfDot, LengthOfFileName;
        	string Extension, ReturnValue = "";
        	
        	// Get file extension
            PositionOfDot = FileName.LastIndexOf(".");
			LengthOfFileName = FileName.Length;
			Extension = FileName.Substring(PositionOfDot + 1, (LengthOfFileName - PositionOfDot) - 1);

            if (Array.IndexOf(ImageAcceptedExtensions, Extension) > -1) {
 				//File is Image
				//If image has thumbnail use it else use the image itself
                Uri ThumbURI = new Uri(Folder + FileName);
				if (File.Exists(ThumbnailsPath + MD5("file://" + ThumbURI.AbsolutePath) + ".png")) {
					ReturnValue = ThumbnailsPath + MD5("file://" + ThumbURI.AbsolutePath) + ".png";
				} else {
					ReturnValue = Folder + FileName;
				}
			} else {
				//File is Video
				Uri ThumbURI = new Uri(Folder + FileName);
				if (File.Exists(ThumbnailsPath + MD5("file://" + ThumbURI.AbsolutePath) + ".png")) {
					ReturnValue = ThumbnailsPath + MD5("file://" + ThumbURI.AbsolutePath) + ".png";
				}
			}
			return ReturnValue;
        }

        static void Main(string[] args){
        	string Folder = "";
        	if (args.Length != 0){
        		int SwitchHelp = Array.IndexOf(args, "--help");
        		int SwitchSystem = Array.IndexOf(args, "--system");
	        	if (SwitchHelp > -1){
	        		Console.WriteLine("Version: SetFolderIcon 0.1.1 05/12/2006");
	        		Console.WriteLine("Copyright: Copyright (C) 2006 John Nikolakis\n");
	        		Console.WriteLine("Usage: SetFolderIcon [options] [folder...] [folder...]\n");
	        		Console.WriteLine("where options include:\n");
	        		Console.WriteLine("  --help\tShows this help message");
	        		Console.WriteLine("  --system\tShows if you have everything you need to run setfoldericon");
	        		
	        		Console.WriteLine("\nStill beta but it works!!!!!");
	        	} else if (SwitchSystem > -1){
	        		Console.WriteLine("Version: SetFolderIcon 0.1.1 05/12/2006");
	        		Console.WriteLine("Copyright: Copyright (C) 2006 John Nikolakis\n");
	        		Console.WriteLine("--system hasn't been implemented yet");
	        	} else {
	        		for (int i = 0; i < args.Length; i++){
	        			Folder = args[i];
	        			string LastLetter = Folder[Folder.Length - 1] + " ";
            			if (String.Compare(LastLetter, "/ ") != 0){
            				Folder = Folder + "/";	
            			}
	        			if (Directory.Exists(Folder)){
	        				MainProgram(Folder);
	        				Console.WriteLine("{0}. {1} : OK", (i + 1), Folder);
	        			} else {
	        			    Console.WriteLine("{0}. {1} : Isn't a valid directory", (i + 1), Folder);
	        			}
	        		}
	        	}
        	} else {
        		Console.WriteLine("Invalid Arguments type --help for help");	
        	}

        }
        
        static void MainProgram(string Folder){
			string[] PhotoNames = {"Folder.jpg", "Folder.png"};
			string HomeFolder = System.Environment.GetEnvironmentVariable("HOME") + "/";
            string ThumbnailsPath = HomeFolder + ".thumbnails/normal/";
            string IconsPath = HomeFolder + ".FolderIcons/";
            string ThumbImage = "thumbs.png";
            string BgImage = "bg.png";
            string XMLFolderPath = HomeFolder + ".nautilus/metafiles/";
            string[] AcceptedExtensions = { "*.jpg", "*.JPG", "*.gif", "*.GIF", "*.png", "*.PNG", "*.jpeg", "*.JPEG", "*.avi", "*.AVI", "*.mpg", "*.MPG", "*.mpeg*", "*.MPEG", "*.mp4*", "*.MP4", "*.mov", "*.MOV"};

            string[] files;
            string[] allfiles = { "" };

            int FileNumber = 0;
            int i, e, NumberOfFiles, RandomNumber;
            
            bool FolderImageExists = false;

            Random RandomClass = new Random();

			// Check if Folder.* exists (If yes use that as folder icon)
			for (e = 0; e < PhotoNames.Length; e++) {
				if (File.Exists(Folder + PhotoNames[e])){
	                //If image has thumbnail use it else use the image itself
	                Uri ThumbURI = new Uri(Folder + PhotoNames[e]);
	                if (File.Exists(ThumbnailsPath + MD5("file://" + ThumbURI.AbsolutePath) + ".png")) {
	                   AppendXmlFile(Folder, IconsPath + MD5("file://" + ThumbURI.AbsolutePath) + ".png", XMLFolderPath);
	                } else {
	                    AppendXmlFile(Folder, Folder + PhotoNames[e], XMLFolderPath);
	                }    
	                FolderImageExists = true;
	                break; 
	           }
			} 
            
            
           // if Folder.* doesn't exist make thumbnails and use them as folder Icons
           if (FolderImageExists == false) {
                //Get number of files with extension (.jpg, .gif, .png, .mpg, .avi, .mpeg, .mp4, .mov   etc.) 
                for (e = 0; e < AcceptedExtensions.Length; e++) {
                    files = Directory.GetFiles(Folder, AcceptedExtensions[e]);
                    for (i = 0; i < files.Length; i++) {
                        Array.Resize(ref allfiles, (FileNumber + 1));
                        allfiles[FileNumber] = System.IO.Path.GetFileName(files[i]);
                        FileNumber++;
                    }
                }
                NumberOfFiles = allfiles.Length;

			
                string[] FolderImages = { "" };
				string ThumbFile;
                if (NumberOfFiles > 4) {
                	// Get four random files
                    int[] RandomNumbers;
                    RandomNumbers = new int[4];
                    for (i = 0; i < 4; i++) {
                        do {
                            do {
                                RandomNumber = RandomClass.Next(NumberOfFiles);
                            } while (Array.IndexOf(RandomNumbers, RandomNumber) > -1);
							
							ThumbFile = FileToUse(allfiles[RandomNumber], Folder, ThumbnailsPath);
							if (ThumbFile != ""){
								Array.Resize(ref FolderImages, (i + 1));
								FolderImages[i] = ThumbFile;
							}
                        } while (ThumbFile == "");
                        RandomNumbers[i] = RandomNumber;
                    }
				} else if (NumberOfFiles > 0) {
					// Use all available files
					for (i = 0; i < NumberOfFiles; i++) {
							ThumbFile = FileToUse(allfiles[i], Folder, ThumbnailsPath);
							if (ThumbFile != ""){
								Array.Resize(ref FolderImages, (i + 1));
								FolderImages[i] = ThumbFile;
							}
					}
 				}

				int NumberOfThumbnailImages = FolderImages.Length;
				if (NumberOfThumbnailImages > 0){
                    //Run ImageMagik commands to create folder.png
                    string arguments = "";
                    string FolderImageName = MD5(Folder) + ".png";
                    System.Diagnostics.Process process1 = new System.Diagnostics.Process();
                    process1.EnableRaisingEvents = false;

                    for (i = 0; i < NumberOfThumbnailImages; i++) {
                        arguments += "\"" + FolderImages[i] + "\" ";
                    }
                    if (NumberOfThumbnailImages > 2){
                    	arguments += " -geometry \"56x56>\"  -tile 2x2 -mode Concatenate -background none \"" + IconsPath + ThumbImage + "\"";
                    } else if (NumberOfThumbnailImages == 2){
						arguments += " -geometry \"56x56>\"  -tile 2x1 -mode Concatenate -background none \"" + IconsPath + ThumbImage + "\"";
                    } else {
                    	arguments += " -geometry \"112x112>\" -mode Concatenate -background none \"" + IconsPath + ThumbImage + "\"";
                    }
                    process1.StartInfo.FileName = "montage";
                    process1.StartInfo.Arguments = arguments;
                    process1.Start();
                    process1.WaitForExit();

                    arguments = " \"" + IconsPath + BgImage + "\" \"" + IconsPath + ThumbImage + "\" -gravity center -composite \"" + IconsPath + FolderImageName + "\"";
                    process1.StartInfo.FileName = "convert";
                    process1.StartInfo.Arguments = arguments;
                    process1.Start();
                    process1.WaitForExit();
                    process1.Close();

                    File.Delete(@IconsPath + ThumbImage);

                    //Edit or Create XML File
                    AppendXmlFile(Folder, IconsPath + FolderImageName, XMLFolderPath);
				}
            }
        }
    }
}
